package JavaGroups;

import java.lang.reflect.Method;
import java.util.Vector;

public class MethodLookupClos implements MethodLookup {


    int ComputeDistance(Class from, Class to) {
	int    retval=0;
	Class  current=from;
	
	while(current != null) {
	    current=current.getSuperclass();
	    if(current == null)
		break;
	    else if(current.equals(to))
		return ++retval;	   
	    else
		retval++;		
	}
	return 0;
    }



    /* Assume that args is non-null and that method and args are same size.
       Matches arguments with formal parameters according to 'proximity' of argument
       to parameter, e.g. an exact match yields 0, a direct subtype yields 1, and an
       indirect subtype <n> where <n> is the distance (e.g. 3). No match returns -1.
       The best match is the one with the lowest return value */

    int MatchParameters(Method method, Vector args) {
	Class[]  formal_parms=method.getParameterTypes();
	Class    formal_parm;
	Object   arg;
	int      retval=0;
	
	for(int i=0; i < formal_parms.length; i++) {
	    formal_parm=formal_parms[i];
	    arg=args.elementAt(i);

	    if(formal_parm.equals(arg.getClass())) {      // exact match argument with formal parm
		retval+=0;
		continue;
	    }
	    if(formal_parm.isInstance(arg)) {             // is arg a subtype of formal_class ?
		int t=ComputeDistance(arg.getClass(), formal_parm);
		return t;
	    }
	    else                                          // no match
		return -1;
	}
	return retval;
    }


    
    public Method FindMethod(Class target_class, String method_name, Vector args) throws Exception {
	Method     retval=null, method;
	Method[]   methods=target_class.getMethods();
	Vector     matching_methods=new Vector();  // contains all possible matches
	int        num_args=args.size();
	Class      parm_cl;
	Class[]    parm_types;
	int        rank=1000, new_rank=0;



	/* 1. Match by name and number of parameters */

	for(int i=0; i < methods.length; i++) {
	    method=methods[i];
	    if(method.getName().equals(method_name) && 
	       method.getParameterTypes().length == num_args) {
		matching_methods.addElement(method);
		continue;
	    }
	}

	if(matching_methods.size() == 1)
	    return (Method)matching_methods.elementAt(0);
	else if(matching_methods.size() < 1)
	    throw new NoSuchMethodException();
	    	

	/* 2. If this is not enough (more than 1 method matching), match formal parameters 
	   with actual arguments. Discard methods whose arguments cannot be cast to the 
	   formal parameters */

	for(int i=0; i < matching_methods.size(); i++) {
	    method=(Method)matching_methods.elementAt(i);
	    new_rank=MatchParameters(method, args);
	    if(new_rank < 0)
		continue;	
	    if(new_rank <= rank) {  // Discards duplicate methods ! But we don't care ...
		retval=method;
		rank=new_rank;
	    }
	}

	if(retval != null)
	    return retval;
	else 
	    throw new NoSuchMethodException();
    }
}
